<?php

namespace BexioConnectorByHelvy;

/**
 * Handle WooCommerce Webhooks Events
 *
 * @since 0.1.0
 */

defined('ABSPATH') || exit;

class WooCommerceWebhooks
{

	public function __construct()
	{
        add_action('woocommerce_new_order', array($this, 'handle_order_created'), 10, 2);
        add_filter('woocommerce_order_status_changed', array($this, 'handle_order_status_changed'), 10, 3 );
        add_action('woocommerce_subscription_payment_complete', array($this, 'handle_subscription_payment_complete'), 10, 2);
        add_action('woocommerce_subscription_renewal_payment_complete', array($this, 'handle_renewal_payment_complete'), 10, 2);
        add_filter('woocommerce_email_attachments', array($this, 'attach_bexio_pdf_to_wc_emails'), 10, 3 );

	}

    function handle_order_status_changed($order_id, $old_status, $new_status)
    {
        $order = wc_get_order($order_id);

        $this->handle_order_created($order_id, $order);

        // If "sync on order created", bypass this. Otherwise, sync only if the order is in a syncable state.
        error_log("Order ID: " . $order_id);
        error_log("Old status: " . $old_status);
        error_log("New status: " . $new_status);

        // // Default statuses that trigger order sync: pending, on-hold, processing, completed
        // // if (get_option(AdminAPISettings::$prefix . 'sync_on_order_created')) {
        // //     $this->handle_order_created($order_id, $order);
        // // } else {
        // //     if (in_array($new_status, array('pending', 'on-hold', 'processing', 'completed'))) {
        // //         WCToBexio::instance()->sync_order($order);
        // //         $order = wc_get_order($order_id); // Re-fetch the order to get the updated meta
        // //         WCToBexio::instance()->sync_order_fulfillment($order);
        // //         $order = wc_get_order($order_id); // Re-fetch the order to get the updated meta
        // //     }
        // // }

        // // Default statuses that trigger payment sync: processing, completed
        // if (in_array($new_status, array('processing', 'completed'))) {
        //     WCToBexio::instance()->sync_order_payment($order);
        //     $order = wc_get_order($order_id); // Re-fetch the order to get the updated meta
        // }

        // // Default statuses that trigger fulfillment sync: completed
        // if ($new_status === 'completed') {
        //     WCToBexio::instance()->sync_order_fulfillment($order);
        // }
    }

    /**
     * Triggered when a new order is created
     * @param $order_id int Order ID
     * @param $order array Order
     * @return void
     */
	function handle_order_created($order_id, $order) {
        // Each sync step is independent and can be called separately (and will check the order status)

        WCToBexio::instance()->sync_order($order);
        $order = wc_get_order($order_id); // Re-fetch the order to get the updated meta
        
        WCToBexio::instance()->sync_order_create_delivery($order, true);
        $order = wc_get_order($order_id);

        if ($order->get_total() != 0) {
            // Sync order's invoice
            WCToBexio::instance()->sync_order_create_invoice($order);
            $order = wc_get_order($order_id);

            // Issue order's invoice
            WCToBexio::instance()->sync_order_issue_invoice($order);
            $order = wc_get_order($order_id);

            // Sync order's payment (to bexio invoice)
            WCToBexio::instance()->sync_order_payment($order);
            $order = wc_get_order($order_id);
        }

        // Sync order's fulfillment (mark delivery as done/fulfilled in bexio)
        WCToBexio::instance()->sync_order_fulfillment($order);
    }

    /**
     * Handle subscription payment complete (which creates the first order of the subscription)
     * @param WC_Subscription $subscription Subscription object
     * @return void
     */
    public function handle_subscription_payment_complete($subscription) {
        $order = $subscription->get_last_order();
        if ($order) {
            if (is_int($order)) {
                $order = wc_get_order($order);
            }
            $this->handle_order_created($order->get_id(), $order);
        }
    }

    /**
     * Handle subscription renewal payment complete (which creates the renewal order of the subscription)
     * @param WC_Subscription $subscription Subscription object
     * @param WC_Order $last_order Last order object
     * @return void
     */
    public function handle_renewal_payment_complete($subscription, $last_order) {
        $this->handle_order_created($last_order->get_id(), $last_order);
    }

    /**
     * Attach the Bexio invoice/order PDF to WooCommerce emails
     * @param $attachments array Array of attachments
     * @param $email_id string Email ID
     * @param $order WC_Order Order object
     * @return array Array of attachments
     */
    function attach_bexio_pdf_to_wc_emails( $attachments, $email_id, $order ) {
        if (!$order instanceof \WC_Order) {
            return $attachments;
        }
        
        $attach_invoice_statuses = get_option(AdminAPISettings::$prefix . 'attach_bexio_invoice_to_woocommerce_email_statuses', array());
        $attach_order_statuses = get_option(AdminAPISettings::$prefix . 'attach_bexio_order_to_woocommerce_email_statuses', array());
        $attach_for_payment_methods = get_option(AdminAPISettings::$prefix . 'attach_bexio_document_to_woocommerce_email_for_payment_methods', array());

        if (empty($attach_invoice_statuses) && empty($attach_order_statuses)) {
            return $attachments;
        }

        if (is_array($attach_for_payment_methods) && !empty($attach_for_payment_methods)) {
            $payment_method = $order->get_payment_method();
            if (!in_array(strtolower($payment_method), array_map('strtolower', $attach_for_payment_methods))) {
                return $attachments;
            }
        } // else all payment methods are allowed

        // Check if the email ID is in the list of statuses to attach the order
        if (is_array($attach_order_statuses) && in_array($email_id, $attach_order_statuses)) {
            // Check for the order ID with a maximum of 10 attempts, one per second
            $max_attempts = 10;
            $attempts = 0;
            $order_id = '';
            while ($attempts < $max_attempts) {
                $order_id = $order->get_meta(AdminAPISettings::$prefix . 'bexio_order_id');
                if ($order_id) {
                    break;
                }
                $attempts++;
                sleep(1);
            }

            // Order not available
            if (!$order_id) {
                error_log("No order ID found after $max_attempts attempts for order " . $order->get_id());
                return $attachments;
            }

            $order_pdf = API::instance()->get_order_pdf($order_id);
            if ($order_pdf instanceof \WP_Error) {
                error_log("Not able to retrieve order PDF for order " . $order->get_id());
                return $attachments;
            }

            // Fetch the order in base64 format from an API
            $order_base64 = $order_pdf["content"];
            if ( $order_base64 ) {
                $decoded_pdf = base64_decode($order_base64);
                if ($decoded_pdf === false) {
                    error_log('Failed to decode base64 PDF order for order ID ' . $order->get_id());
                    return $attachments;
                }

                // Create a temporary file with a custom filename
                $upload_dir = wp_upload_dir();
                $temp_file_path = tempnam($upload_dir['basedir'], 'order-' . $order->get_id() . '-');
                $custom_temp_file_path = $temp_file_path . '.pdf';
                rename($temp_file_path, $custom_temp_file_path);

                $temp_file = fopen($custom_temp_file_path, 'w');
                $file_written = fwrite($temp_file, $decoded_pdf);
                if ($file_written === false) {
                    error_log('Failed to write the decoded PDF to the temporary file for order ID ' . $order->get_id());
                    return $attachments;
                }
                fclose($temp_file);

                // Attach the temporary file to the email with a custom filename
                add_action('phpmailer_init', function($phpmailer) use ($custom_temp_file_path, $order) {
                    $phpmailer->addAttachment($custom_temp_file_path, 'order-' . $order->get_id() . '.pdf');
                    register_shutdown_function(function() use ($custom_temp_file_path) {
                        unlink($custom_temp_file_path);
                    });
                });
            }
        }
        
        // Check if the email ID is in the list of statuses to attach the invoice
        if (is_array($attach_invoice_statuses) && in_array($email_id, $attach_invoice_statuses)) {
            // Check for the invoice ID with a maximum of 10 attempts, one per second
            $max_attempts = 10;
            $attempts = 0;
            $invoice_id = '';
            while ($attempts < $max_attempts) {
                $invoice_id = $order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_id');
                if ($invoice_id) {
                    break;
                }
                $attempts++;
                sleep(1);
            }

            // Invoice not available
            if (!$invoice_id) {
                error_log("No invoice ID found after $max_attempts attempts for order " . $order->get_id());
                return $attachments;
            }

            $invoice_pdf = API::instance()->get_invoice_pdf($invoice_id);
            if ($invoice_pdf instanceof \WP_Error) {
                error_log("Not able to retrieve invoice PDF for order " . $order->get_id());
                return $attachments;
            }

            // Fetch the invoice in base64 format from an API
            $invoice_base64 = $invoice_pdf["content"];
            if ( $invoice_base64 ) {
                $decoded_pdf = base64_decode($invoice_base64);
                if ($decoded_pdf === false) {
                    error_log('Failed to decode base64 PDF invoice for order ID ' . $order->get_id());
                    return $attachments;
                }

                // Create a temporary file with a custom filename
                $upload_dir = wp_upload_dir();
                $temp_file_path = tempnam($upload_dir['basedir'], 'invoice-' . $order->get_id() . '-');
                $custom_temp_file_path = $temp_file_path . '.pdf';
                rename($temp_file_path, $custom_temp_file_path);

                $temp_file = fopen($custom_temp_file_path, 'w');
                $file_written = fwrite($temp_file, $decoded_pdf);
                if ($file_written === false) {
                    error_log('Failed to write the decoded PDF to the temporary file for order ID ' . $order->get_id());
                    return $attachments;
                }
                fclose($temp_file);

                // Attach the temporary file to the email with a custom filename
                add_action('phpmailer_init', function($phpmailer) use ($custom_temp_file_path, $order) {
                    $phpmailer->addAttachment($custom_temp_file_path, 'invoice-' . $order->get_id() . '.pdf');
                    register_shutdown_function(function() use ($custom_temp_file_path) {
                        unlink($custom_temp_file_path);
                    });
                });
            }
        }
    
        return $attachments;
    }
    
}
